/*
 * *********************************************************
 * Copyright (c) 2009 - 2010, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: 03.08.2010
 * Author(s):
 * Oliver Steinbrecher
 * Daniel Waigand
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.data.frames;

import edu.dhbw.mannheim.tigers.sumatra.model.data.airecord.IRecordFrame;


/**
 * This class is a simple container for all information the AI gathers during its processes for one {@link WorldFrame}
 * 
 * @author Oliver Steinbrecher, Daniel Waigand, Nicolai Ommer <nicolai.ommer@gmail.com>
 */
public class AIInfoFrame extends MetisAiFrame implements IRecordFrame
{
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	
	/** frames per second (based on last frames) */
	private final float	fps;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	
	/**
	 * @param metisFrame
	 * @param fps
	 */
	public AIInfoFrame(final MetisAiFrame metisFrame, final float fps)
	{
		super(metisFrame, metisFrame.getTacticalField());
		this.fps = fps;
	}
	
	
	/**
	 * Providing a <strong>shallow</strong> copy of original (Thus collections are created, but filled with the same
	 * values (instead of copying these values, too))
	 * 
	 * @param original
	 */
	public AIInfoFrame(final AIInfoFrame original)
	{
		super(original, original.getTacticalField());
		fps = original.fps;
	}
	
	
	@Override
	public void cleanUp()
	{
		super.cleanUp();
	}
	
	
	/**
	 * @return the fps
	 */
	public float getFps()
	{
		return fps;
	}
	
	
	@Override
	public boolean isPersistable()
	{
		return false;
	}
}
