/*
 * *********************************************************
 * Copyright (c) 2009 - 2013, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: Apr 7, 2013
 * Author(s): Daniel Andres <andreslopez.daniel@gmail.com>
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators;

import java.util.Arrays;
import java.util.List;

import edu.dhbw.mannheim.tigers.sumatra.model.data.modules.ai.ETeam;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.general.BallLeftFieldCalc;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.general.BallPossessionCalc;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.general.BotLastTouchedBallCalc;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.general.BotToBallDistanceCalc;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.general.GameStateCalc;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.ai.metis.calculators.general.PossibleGoalCalc;
import edu.dhbw.mannheim.tigers.sumatra.util.IInstanceableEnum;
import edu.dhbw.mannheim.tigers.sumatra.util.InstanceableClass;
import edu.dhbw.mannheim.tigers.sumatra.util.InstanceableParameter;


/**
 * This Enum contains all calculators by name
 * 
 * @author Daniel Andres <andreslopez.daniel@gmail.com>
 */
public enum ECalculator implements IInstanceableEnum
{
	/**  */
	GAME_STATE(new InstanceableClass(GameStateCalc.class)),
	/** */
	BOT_TO_BALL_DISTANCE_TIGERS(new InstanceableClass(BotToBallDistanceCalc.class, new InstanceableParameter(
			ETeam.class, "team", ETeam.TIGERS.name()))),
	/** */
	BOT_TO_BALL_DISTANCE_OPPONENTS(new InstanceableClass(BotToBallDistanceCalc.class, new InstanceableParameter(
			ETeam.class, "team", ETeam.OPPONENTS.name()))),
	/** */
	BALL_POSSESSION(new InstanceableClass(BallPossessionCalc.class), BOT_TO_BALL_DISTANCE_TIGERS,
			BOT_TO_BALL_DISTANCE_OPPONENTS),
	/** */
	POSSIBLE_GOAL(new InstanceableClass(PossibleGoalCalc.class)),
	/** */
	BOT_LAST_TOUCHED_BALL(new InstanceableClass(BotLastTouchedBallCalc.class)),
	/**  */
	BALL_LEFT_FIELD(new InstanceableClass(BallLeftFieldCalc.class)), ;
	
	private final boolean				initiallyActive;
	private final long					timeRateMs;
	private final InstanceableClass	impl;
	private final List<ECalculator>	dependencies;
	
	
	private ECalculator(final InstanceableClass impl, final ECalculator... dependencies)
	{
		this(impl, 0, true, dependencies);
	}
	
	
	private ECalculator(final InstanceableClass impl, final long timeRateMs, final ECalculator... dependencies)
	{
		this(impl, timeRateMs, true, dependencies);
	}
	
	
	private ECalculator(final InstanceableClass impl, final boolean initiallyActive, final ECalculator... dependencies)
	{
		this(impl, 0, initiallyActive, dependencies);
	}
	
	
	private ECalculator(final InstanceableClass impl, final long timeRateMs, final boolean initiallyActive,
			final ECalculator... dependencies)
	{
		this.impl = impl;
		this.timeRateMs = timeRateMs;
		this.initiallyActive = initiallyActive;
		this.dependencies = Arrays.asList(dependencies);
	}
	
	
	/**
	 * @return the initiallyActive
	 */
	public final boolean isInitiallyActive()
	{
		return initiallyActive;
	}
	
	
	/**
	 * @return the timeRateMs
	 */
	public final long getTimeRateMs()
	{
		return timeRateMs;
	}
	
	
	/**
	 * @return the impl
	 */
	public final InstanceableClass getImpl()
	{
		return impl;
	}
	
	
	/**
	 * @return the dependencies
	 */
	public final List<ECalculator> getDependencies()
	{
		return dependencies;
	}
	
	
	@Override
	public InstanceableClass getInstanceableClass()
	{
		return impl;
	}
}
